require "defines"
require "util"
--[[ globs and functions defined

glob.AlyxRYS_MasterList
	acts as a map, with a category label as the key and an array of pollution readers as the value
glob.AlyxRYS_CurrentCategory
	holds the category currently selected

-function AlyxRYS_Init(currentPlayer)
	Initializes globs and creates gui for player, should run on oninit,onload,onplayercreated
-function AlyxRYS_UpdateCurrentCategory(category)
	Updates the current category readers will be added to, if the category doesn't exist, calls AlyxRYS_CategoryAdd(category)

-function AlyxRYS_CreateGUI(currentPlayer)
	Self explanatory
-function AlyxRYS_AddCategoryToGUI(category)
-function AlyxRYS_UpdateValuesInGUI(categoryToUpdate)
	Updates the displayed pollution value in the GUI
	category parameter is optional, if not specified it will update all categories, otherwise just the category provided

-function AlyxRYS_CategoryAdd(category)
	Creates the provided category and adds it to glob.AlyxRYS_MasterList
-function AlyxRYS_CategoryRemove(category)
	Removes the provided category, if the category is not "default" then it will add any readers in that category to the "default" category, creating the category if necessary. If the category to delete is "default," then it just removes everything.
-function AlyxRYS_ReaderAdd(reader,category)
	Adds reader to category, doesn't store by key, so they must be accessed incrementally
-function AlyxRYS_ReaderRemove(readerIndex,category)
	Because readers aren't stored with keys, most efficient way to delete them is by index.
--]]

---------------------------------------
-- Main Logic Loop
---------------------------------------
game.onevent(defines.events.ontick, function(event)
	if (game.tick / 60) % 10 == 0 then
		AlyxRYS_UpdateValuesInGUI()
	end

end)
---------------------------------------
---------------------------------------


---------------------------------------
-- Initializations
---------------------------------------
function AlyxRYS_Init(currentPlayer)
	if glob.AlyxRYS_MasterList == nil then
		glob.AlyxRYS_MasterList = {}
	end
	AlyxRYS_CreateGUI(currentPlayer)
	
	if glob.AlyxRYS_CurrentCategory == nil then
		AlyxRYS_UpdateCurrentCategory("default")
	end
end

game.oninit(function()
	for index,player in pairs(game.players) do
		if player ~= nil then
			AlyxRYS_Init(player)
		end
	end
	
end)
game.onload(function()
	for index,player in pairs(game.players) do
		if player ~= nil then
			AlyxRYS_Init(player)
		end
	end
end)
game.onevent(defines.events.onplayercreated, function(event)
	AlyxRYS_Init(game.getplayer(event.playerindex))
end)
---------------------------------------
---------------------------------------


---------------------------------------
-- Category and Reader Functions
---------------------------------------
function AlyxRYS_ReaderAdd(reader,category)
	category = category or "Uncategorised"
	
	if glob.AlyxRYS_MasterList[category] == nil then
		AlyxRYS_CategoryAdd(category)
	end
	table.insert(glob.AlyxRYS_MasterList[category], reader)
end

function AlyxRYS_ReaderRemove(readerIndex,category)
	category = category or "Uncategorised"

	local tempCategory = glob.AlyxRYS_MasterList[category]
	if tempCategory[readerIndex].valid then
		table.remove(glob.AlyxRYS_MasterList[category], reader)
	end
end

---------

function AlyxRYS_CategoryAdd(category)

	if glob.AlyxRYS_MasterList[category] == nil then
		glob.AlyxRYS_MasterList[category] = {}
		AlyxRYS_AddCategoryToGUI(category)
		AlyxRYS_UpdateValuesInGUI(category)

	end

end

function AlyxRYS_CategoryRemove(category)

	if glob.AlyxRYS_MasterList[category] ~= nil then
		if category ~= "default" then
			for index, reader in pairs(glob.AlyxRYS_MasterList[category]) do
				if reader.valid then
					AlyxRYS_ReaderAdd(reader,"default")
				end
			end
			AlyxRYS_UpdateValuesInGUI("default")
		end
		for index, player in pairs(game.players) do
			player.gui.top.AlyxReader[category].destroy()
			
		end
		glob.AlyxRYS_MasterList[category] = nil
	end
end
---------------------------------------
---------------------------------------


---------------------------------------
-- GUI functions
---------------------------------------
game.onevent(defines.events.onguiclick, function(event)

	if event.element.name == "selectSet" then
		AlyxRYS_UpdateCurrentCategory(game.player.gui.top.AlyxReader.inputField.text)
		game.player.gui.top.AlyxReader.inputField.text = ""
		
	elseif event.element.name == "selectRemove" then
		AlyxRYS_CategoryRemove(game.player.gui.top.AlyxReader.inputField.text)
		game.player.gui.top.AlyxReader.inputField.text = ""
		
	end
	
end)
---------

function AlyxRYS_CreateGUI(player)

	if player.gui.top.AlyxReader ~= nil then
		player.gui.top.AlyxReader.destroy()
	end

	player.gui.top.add{type="frame", name="AlyxReader", direction="vertical"}
	player.gui.top.AlyxReader.add{type="label", name="label", caption="Pollution Readers:"}
	player.gui.top.AlyxReader.add{type="label", name="currentCategory", caption="Current category: nil"}
	if glob.AlyxRYS_CurrentCategory ~= nil then
		player.gui.top.AlyxReader["currentCategory"].caption = "Current category: " .. glob.AlyxRYS_CurrentCategory
	end
	player.gui.top.AlyxReader.add{type="textfield", name="inputField"}
	player.gui.top.AlyxReader.add{type="flow", name="selectionButtons", direction="horizontal"}
	player.gui.top.AlyxReader.selectionButtons.add{type="button", name="selectSet", caption="set"}
	player.gui.top.AlyxReader.selectionButtons.add{type="button", name="selectRemove", caption="remove"}

	for key, value in pairs(glob.AlyxRYS_MasterList) do
		AlyxRYS_AddCategoryToGUI(key, player)
	end
	
end
---------

function AlyxRYS_AddCategoryToGUI(category, currentPlayer)

	if currentPlayer == nil then
		for index, player in pairs(game.players) do
			if player ~= nil then
				local currentGUI = player.gui.top.AlyxReader.add{type="flow", name=category, direction="horizontal"}
				currentGUI.style.minimalwidth = 100
				currentGUI.add{type="label", name="label", caption=category ..": "}
				currentGUI.add{type="label", name="pollutionValue", caption="Not yet polled"}
			end
		end
		
	else
		local currentGUI = currentPlayer.gui.top.AlyxReader.add{type="flow", name=category, direction="horizontal"}
		currentGUI.style.minimalwidth = 100
		currentGUI.add{type="label", name="label", caption=category ..": "}
		currentGUI.add{type="label", name="pollutionValue", caption="Not yet polled"}
		
	end
end
---------

function AlyxRYS_UpdateValuesInGUI(categoryToUpdate)

	if categoryToUpdate == nil then
		for key, category in pairs(glob.AlyxRYS_MasterList) do
			local tempSum = 0
			local tempNum = 0
			for index, reader in ipairs(category) do
				if reader.valid then
					tempSum = tempSum + game.getpollution(reader.position)
					tempNum = tempNum + 1
					
				else
					AlyxRYS_ReaderRemove(index, key)

				end
			end
			if tempNum ~= 0 then
				local pollutionValue = math.ceil(tempSum / tempNum)
				for index,player in pairs(game.players) do
					if player ~= nil then
						local guiCategory = player.gui.top.AlyxReader[key]
						if guiCategory ~= nil then
							guiCategory.pollutionValue.caption = pollutionValue
						end
					end
				end
				
			else
				for index,player in pairs(game.players) do
					if player ~= nil then
						local guiCategory = player.gui.top.AlyxReader[key]
						if guiCategory ~= nil then
							guiCategory.pollutionValue.caption = "No readers"
						end
					end
				end
				
			end
		end
	
	elseif glob.AlyxRYS_MasterList[categoryToUpdate] ~= nil then
		local tempSum = 0
		local tempNum = 0
		for index, reader in ipairs(glob.AlyxRYS_MasterList[categoryToUpdate]) do
			if reader.valid then
				tempSum = tempSum + game.getpollution(reader.position)
				tempNum = tempNum + 1
				
			else
				AlyxRYS_ReaderRemove(index, categoryToUpdate)

			end
		end
		if tempNum ~= 0 then
			local pollutionValue = math.ceil(tempSum / tempNum)
			for index,player in pairs(game.players) do
				if player ~= nil then
					local guiCategory = player.gui.top.AlyxReader[categoryToUpdate]
					if guiCategory ~= nil then
						guiCategory.pollutionValue.caption = pollutionValue
					end
				end
			end
			
		else
			for index,player in pairs(game.players) do
				if player ~= nil then
					local guiCategory = player.gui.top.AlyxReader[categoryToUpdate]
					if guiCategory ~= nil then
						guiCategory.pollutionValue.caption = "No readers"
					end
				end
			end
			
		end
	
	end
	
	
end
---------------------------------------
---------------------------------------


---------------------------------------
-- Miscellaneous
---------------------------------------
game.onevent(defines.events.onbuiltentity, function(event)

	if event.createdentity.name == "pollution-reader" then
		AlyxRYS_ReaderAdd(event.createdentity, glob.AlyxRYS_CurrentCategory)
		
	end
end)
---------

function AlyxRYS_UpdateCurrentCategory(category)
	category = category or "Unrecognised"

	glob.AlyxRYS_CurrentCategory = category
	for index,player in pairs(game.players) do
		if player ~= nil and player.gui.top.AlyxReader ~= nil then
			player.gui.top.AlyxReader["currentCategory"].caption = "Current category: " .. category
		end
	end
	
	if glob.AlyxRYS_MasterList[category] == nil then
		AlyxRYS_CategoryAdd(category)
	end
	
end
---------------------------------------
---------------------------------------